//
// This file is part of LiRE, a free, experimental library for
// efficient testing of determinism and membership for regular
// expressions.
//
// LiRE is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, version 3 of the License, or any
// later version.
//
// LiRE is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LiRE. If not, see <http://www.gnu.org/licenses/>.
//
// Authors: Sławek Staworko <staworko@gmail.com>
//          Benoît Groz <benoit.groz@yahoo.fr>
//          Sebastian Maneth <sebastian.maneth@gmail.com>
// Date: 20 June 2011.
//

#include "recognizer.h"
#include "error.h"
#include "utils.h"


struct EmptyWordRecognizer : Recognizer
{
   bool empty_word;

   EmptyWordRecognizer() : Recognizer(new Alphabet()) {this->empty_word = true;};
   ~EmptyWordRecognizer() {this->alphabet->release();};
   void consume(int symbol_id) {this->empty_word = false;};
   bool accepting() {return this->empty_word;};
};


template<bool val> struct ConstRecognizer : Recognizer
{
   ConstRecognizer() : Recognizer(new Alphabet()) {};
   ~ConstRecognizer() {this->alphabet->release();};
   void consume(int symbol_id) {};
   bool accepting() {return val;};
};

const Recognizer* Recognizer::EMPTY = new ConstRecognizer<false>();
const Recognizer* Recognizer::UNIVERSAL = new ConstRecognizer<true>();

Recognizer* Recognizer::getEmptyWordRecognizer()
{
   return new EmptyWordRecognizer();
}

Recognizer::Recognizer(Alphabet* alphabet)
{
   this->alphabet = alphabet->retain();
}

Recognizer::~Recognizer()
{
   this->alphabet->release();
}

void Recognizer::consume(char* symbol)
{
   ASSERT(symbol != NULL);

   this->consume(this->alphabet->find_id(symbol));
}

void Recognizer::consume(Vector<int>& symbol_id)
{
   ASSERT(! symbol_id.null());
   
   for (int i = 0; i < symbol_id.size; i++) {
      this->consume(symbol_id[i]);
   }
}

void Recognizer::consume(Vector<char*>& symbol)
{
   ASSERT(! symbol.null());
   Vector<int> symbol_id;

   this->alphabet->translate(symbol,symbol_id);
   this->consume(symbol_id);
}

