//
// This file is part of LiRE, a free, experimental library for
// efficient testing of determinism and membership for regular
// expressions.
//
// LiRE is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, version 3 of the License, or any
// later version.
//
// LiRE is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LiRE. If not, see <http://www.gnu.org/licenses/>.
//
// Authors: Sławek Staworko <staworko@gmail.com>
//          Benoît Groz <benoit.groz@yahoo.fr>
//          Sebastian Maneth <sebastian.maneth@gmail.com>
// Date: 20 June 2011.
//
#ifndef LiRE_RECOGNIZER_H_
#define LiRE_RECOGNIZER_H_

#include "alphabet.h"
#include "vector.h"

/**
 * A uniform interface for recognizing a string.
 */
struct Recognizer
{

   static const Recognizer* UNIVERSAL;
   static const Recognizer* EMPTY;
   static Recognizer* getEmptyWordRecognizer();

   Alphabet* alphabet;

   Recognizer(Alphabet* alphabet);
   virtual ~Recognizer() = 0;

   virtual bool accepting() = 0;

   virtual void consume(int symbol_id) = 0;
   virtual void consume(char* symbol);
   virtual void consume(Vector<int>& symbol_id);
   virtual void consume(Vector<char*>& symbol);

   // -----------------------------------------------------------------------
   //  Unimplemented constructors and operators
   // -----------------------------------------------------------------------
   Recognizer(const Recognizer&);
   Recognizer& operator=(const Recognizer&);
};

#endif // LiRE_RECOGNIZER_H_
