//
// This file is part of LiRE, a free, experimental library for
// efficient testing of determinism and membership for regular
// expressions.
//
// LiRE is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, version 3 of the License, or any
// later version.
//
// LiRE is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LiRE. If not, see <http://www.gnu.org/licenses/>.
//
// Authors: Sławek Staworko <staworko@gmail.com>
//          Benoît Groz <benoit.groz@yahoo.fr>
//          Sebastian Maneth <sebastian.maneth@gmail.com>
// Date: 20 June 2011.
//
#ifndef LiRE_ERROR_H_
#define LiRE_ERROR_H_

#include <stdlib.h>
#include "logger.h"

// If the compiler is not GNU compliant
#ifndef __GNUC__
//#error "__GNUC_ undef"
#define __attribute__(x) 
#endif

extern void _LiRE_handle_error(const char* file_name, 
                               const int line_no, 
                               const char* format,...)
   __attribute__((format (printf,3,4)));

extern void _LiRE_handle_assertion_failure(const char* file_name, 
                                           const int line_no, 
                                           const char* format,...)
   __attribute__((format (printf,3,4)));

// Assertions
#ifndef LiRE_DEBUG_OFF_
#define ASSERT(cond,...)                                            \
   do {                                                             \
      if (!(cond)) {                                                \
         _LiRE_handle_assertion_failure(__FILE__, __LINE__,         \
                                        "Assertion \""              \
                                        #cond "\" failed! "         \
                                        __VA_ARGS__);               \
      }                                                             \
   } while(0)
#else
#define ASSERT(cond,...) do{} while(0)
#endif

// General error handler.
#define ERROR(...)                                                  \
   do {                                                             \
      _LiRE_handle_error(__FILE__, __LINE__, __VA_ARGS__);          \
      exit(1);                                                      \
   } while(0)

#endif // LiRE_ERROR_H_
