//
// This file is part of LiRE, a free, experimental library for
// efficient testing of determinism and membership for regular
// expressions.
//
// LiRE is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, version 3 of the License, or any
// later version.
//
// LiRE is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LiRE. If not, see <http://www.gnu.org/licenses/>.
//
// Authors: Sławek Staworko <staworko@gmail.com>
//          Benoît Groz <benoit.groz@yahoo.fr>
//          Sebastian Maneth <sebastian.maneth@gmail.com>
// Date: 20 June 2011.
//
#ifndef LiRE_LOGGER_H_
#define LiRE_LOGGER_H_

#include <stdarg.h>

// If the compiler is not GNU compliant
#ifndef __GNUC__
//#error "__GNUC_ undef"
#define __attribute__(x) 
#endif

struct Logger
{
   enum LogLevel {FORCE_LEVEL    = 0, 
                  CRITICAL_LEVEL = 1, 
                  ERROR_LEVEL    = 2, 
                  NOTICE_LEVEL   = 3, 
                  INFO_LEVEL     = 4, 
                  DEBUG_LEVEL    = 5};

   LogLevel level;
   void msg (LogLevel level, const char* file_name, int line_no,
             const char* format, va_list ap);

   void msg         (LogLevel level, const char* file_name, int line_no,
                     const char* format,...) __attribute__((format (printf,5,6)));
   void force       (const char* file_name, int line_no,
                     const char* format,...) __attribute__((format (printf,4,5)));
   void critical    (const char* file_name, int line_no,
                     const char* format,...) __attribute__((format (printf,4,5)));
   void error       (const char* file_name, int line_no,
                     const char* format,...) __attribute__((format (printf,4,5)));
   void notice      (const char* file_name, int line_no,
                     const char* format,...) __attribute__((format (printf,4,5)));
   void info        (const char* file_name, int line_no,
                     const char* format,...) __attribute__((format (printf,4,5)));
   void debug       (const char* file_name, int line_no,
                     const char* format,...) __attribute__((format (printf,4,5)));

   static Logger* getLogger();
private:
   Logger();
};

#define FORCE_LOG(...) Logger::getLogger()->force(__FILE__,__LINE__,__VA_ARGS__)
#define CRITICAL_LOG(...) Logger::getLogger()->critical(__FILE__,__LINE__,__VA_ARGS__)
#define ERROR_LOG(...) Logger::getLogger()->error(__FILE__,__LINE__,__VA_ARGS__)
#define NOTICE_LOG(...) Logger::getLogger()->notice(__FILE__,__LINE__,__VA_ARGS__)
#define INFO_LOG(...) Logger::getLogger()->info(__FILE__,__LINE__,__VA_ARGS__)
#define DEBUG_LOG(...) Logger::getLogger()->debug(__FILE__,__LINE__,__VA_ARGS__)

#ifndef LiRE_DEBUG_OFF_
#define DEBUG(...) Logger::getLogger()->debug(__FILE__,__LINE__,__VA_ARGS__)
#else
#define DEBUG(...) do {} while (false)
#endif

#endif // LiRE_LOGGER_H_
