//
// This file is part of LiRE, a free, experimental library for
// efficient testing of determinism and membership for regular
// expressions.
//
// LiRE is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, version 3 of the License, or any
// later version.
//
// LiRE is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LiRE. If not, see <http://www.gnu.org/licenses/>.
//
// Authors: Sławek Staworko <staworko@gmail.com>
//          Benoît Groz <benoit.groz@yahoo.fr>
//          Sebastian Maneth <sebastian.maneth@gmail.com>
// Date: 20 June 2011.
//
#include "tree.h"

Node::Node()
   : color_id(-1)
   , type(UNK) 
   , nullable(false)
   , left(NULL)
   , right(NULL)
   , inter_begin(-1)
   , inter_end(-1)
   , parent(NULL)
   , root_first(NULL)
   , root_last(NULL)
   , star(NULL)
   , rfp(NULL)
   , next_left(NULL)
   , skel_leaf(NULL)
   , some_first_pos(NULL)
   , n_ref(NULL)
   , next_top(NULL)
   , id(-1)
   , root_first_id(-1)
   , root_last_id(-1)
   , star_id(-1)
   , pos_id(-1)
   , root_first_pos_id(-1)
   , root_last_pos_id(-1)
   , color_pos_id(-1)
   , rfp_id(-1)
   , rfp_pos_id(-1)
{
}

Node::~Node()
{
//    if (this->left != NULL) {
//       delete left;
//    }
//    if (this->right != NULL) {
//       delete right;
//    }
}

bool Node::is_left_child() {
   return parent != NULL and parent->left == this;
}

bool Node::is_right_child() {
   return parent != NULL and parent->right == this;
}

bool Node::is_root_first() {
   return root_first == this;
}

bool Node::is_root_last() {
   return root_last == this;
}

bool Node::is_root() {
   return type == ROOT;
}

bool Node::is_rfp() {
   return (is_concat() or type == ROOT) and right->is_root_first();
}

bool Node::is_leaf() {
   return left == NULL and right == NULL;
}

bool Node::is_pos() {
   return type == POS;
}

bool Node::is_epsilon() {
   return type == EPSILON;
}

bool Node::is_concat() {
   return type == CONCAT;
}

bool Node::is_union() {
   return type == UNION;
}

bool Node::is_star() {
   return type == STAR;
}

bool Node::is_descendant(Node* n) {
   ASSERT(n != NULL);
   return n->inter_begin <= this->inter_begin  
      and this->inter_end <= n->inter_end;
}

bool Node::is_left_descendant(Node* n) {
   ASSERT(n != NULL);
   return n->left != NULL and this->is_descendant(n->left);
}

bool Node::is_right_descendant(Node* n) {
   ASSERT(n != NULL);
   return n->right != NULL and this->is_descendant(n->right);
}

bool Node::is_ancestor(Node* n) {
   ASSERT(n != NULL);
   return n->is_descendant(this);
}

// #include <stdio.h>
// void Node::print()
// {
//    printf("N[id=%i, type=%i, color=%i]\n",id,type,color_id);
// }
