%{
//
// This file is part of LiRE, a free, experimental library for
// efficient testing of determinism and membership for regular
// expressions.
//
// LiRE is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, version 3 of the License, or any
// later version.
//
// LiRE is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LiRE. If not, see <http://www.gnu.org/licenses/>.
//
// Authors: Sławek Staworko <staworko@gmail.com>
//          Benoît Groz <benoit.groz@yahoo.fr>
//          Sebastian Maneth <sebastian.maneth@gmail.com>
// Date: 20 June 2011.
//

#include "regexpr.h"
#include "alphabet.h"
#include <stdio.h>


Alphabet*      lire_regexpr_yy_alphabet = NULL;
RegExpr::Node* lire_regexpr_yy_result = NULL;

int yylex (void);
void yyerror(char* msg)
{
   printf("%s",msg);
   exit(1);
}
  
%}

%union {
   char*          name;
   RegExpr::Node* expr;
}

%token OP CP BAR_T COMMA_T AST_T OPT_T PLUS_T NAME_T EMPTY_T

%left COMMA_T
%left BAR_T 
%nonassoc OPT_T PLUS_T AST_T

%start reg_expr

%type <name> NAME_T
%type <expr> expr reg_expr

%%
reg_expr : expr {
    lire_regexpr_yy_result=$1;
    $$=lire_regexpr_yy_result; 
}

expr : NAME_T {
          RegExpr::Node* n = new RegExpr::Node(RegExpr::LABEL);
          String label($1,true);
          n->label_id = lire_regexpr_yy_alphabet->find_id(label);
          $$=n;
       }
     | expr BAR_T expr {
          $$ = new RegExpr::Node(RegExpr::UNION);
          $$->left = $1;
          $$->right = $3;
       }
     | expr COMMA_T expr {
          $$ = new RegExpr::Node(RegExpr::CONCAT);
          $$->left = $1;
          $$->right = $3;
       }
     | expr AST_T {
          $$ = new RegExpr::Node(RegExpr::STAR);
          $$->left = $1;
       }
     | expr PLUS_T {
          $$ = new RegExpr::Node(RegExpr::CONCAT);
          $$->left = $1;
          $$->right = new RegExpr::Node(RegExpr::STAR);
          $$->right->left = $1;
       }
     | expr OPT_T {
          $$ = new RegExpr::Node(RegExpr::OPTION);
          $$->left = $1;
       }
     | EMPTY_T {
          $$ = new RegExpr::Node(RegExpr::EPSILON);
       }
     | OP expr CP {
          $$ = $2;
       }
%%
