//
// This file is part of LiRE, a free, experimental library for
// efficient testing of determinism and membership for regular
// expressions.
//
// LiRE is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, version 3 of the License, or any
// later version.
//
// LiRE is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LiRE. If not, see <http://www.gnu.org/licenses/>.
//
// Authors: Sławek Staworko <staworko@gmail.com>
//          Benoît Groz <benoit.groz@yahoo.fr>
//          Sebastian Maneth <sebastian.maneth@gmail.com>
// Date: 20 June 2011.
//
#ifndef LiRE_SKEL_H_
#define LiRE_SKEL_H_

#include "vector.h"
#include "lca.h"

// Forward declaration of Tree and Tree::Node.
struct Node;
struct Tree;

// An augmented skeleton.
struct Skel 
{
   // A node of an augmented skeleton.
   struct Node 
   {
      int id;
      
      ::Node*  node;    // the underlying tree node
      Node* parent;    // the parent skeleton node
      Node* left;      // the left skeleton node
      Node* right;     // the right skeleton node
      Node* star;      // the closest star node. 
      Node* color_rfp; // the closest rfp node in the color of the
                        // skeleton (whose witness is non-null).
      
      ::Node* witness;   // color witness for rfp nodes.
      ::Node* first_pos; // position of color that belongs to First(node).
      ::Node* next;      // next position of color not descendant of node;
   
      Node(::Node* node);

      ~Node();
      
      void print();
      void print(int ind, bool left);
      void print(int ind);
   };

   Tree* t;
   int size;
   Vector<Node*> node;
   Node* get_node(int i) { return node[i];};
   Node* root;    // the root skeleton node
   int    color_id;
   LCA<Skel,Node>* lca;
   
   Skel(Tree* t, int color_id);
   ~Skel();
   void annotate();
   void build_lca();

   Node* get_lca(Node* x, Node* y);

   void print() { print(0); };
   void print(int ind) {root->print(ind);};

};

#endif // LiRE_SKEL_H_
