//
// This file is part of LiRE, a free, experimental library for
// efficient testing of determinism and membership for regular
// expressions.
//
// LiRE is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, version 3 of the License, or any
// later version.
//
// LiRE is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LiRE. If not, see <http://www.gnu.org/licenses/>.
//
// Authors: Sławek Staworko <staworko@gmail.com>
//          Benoît Groz <benoit.groz@yahoo.fr>
//          Sebastian Maneth <sebastian.maneth@gmail.com>
// Date: 20 June 2011.
//
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/sax/SAXException.hpp>
#include <xercesc/parsers/SAXParser.hpp>
#include <xercesc/framework/XMLValidator.hpp>
#include <xercesc/framework/XMLErrorReporter.hpp>
#include <xercesc/validators/DTD/DTDValidator.hpp>
#include <xercesc/validators/DTD/DTDGrammar.hpp>
#include <xercesc/framework/XMLGrammarDescription.hpp>
#include <xercesc/framework/XMLDTDDescription.hpp>
#include <xercesc/validators/common/Grammar.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include <xercesc/util/OutOfMemoryException.hpp>
#include <xercesc/sax/AttributeList.hpp>
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/sax/SAXException.hpp>
#include <xercesc/util/NameIdPool.hpp>
#include <xercesc/validators/DTD/DTDElementDecl.hpp>
#include <xercesc/framework/XMLContentModel.hpp>
#include <xercesc/validators/common/ContentSpecNode.hpp>
#if defined(XERCES_NEW_IOSTREAMS)
#include <iostream>
#include <fstream>
#else
#include <iostream.h>
#include <fstream.h>
#endif
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <tr1/unordered_map>
#include <limits.h>

#include "error.h"
#include "str.h"
#include "regexpr.h"
#include "glushkov.h"



#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/sax/SAXException.hpp>
#include <xercesc/parsers/SAXParser.hpp>
#include <xercesc/framework/XMLValidator.hpp>
#include <xercesc/framework/XMLErrorReporter.hpp>
#include <xercesc/validators/DTD/DTDValidator.hpp>
#include <xercesc/validators/DTD/DTDGrammar.hpp>
#include <xercesc/framework/XMLGrammarDescription.hpp>
#include <xercesc/framework/XMLDTDDescription.hpp>
#include <xercesc/validators/common/Grammar.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include <xercesc/util/OutOfMemoryException.hpp>
#include <xercesc/sax/AttributeList.hpp>
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/sax/SAXException.hpp>
#include <xercesc/util/NameIdPool.hpp>
#include <xercesc/validators/DTD/DTDElementDecl.hpp>
#include <xercesc/framework/XMLContentModel.hpp>
#include <xercesc/validators/common/ContentSpecNode.hpp>
#if defined(XERCES_NEW_IOSTREAMS)
#include <iostream>
#include <fstream>
#else
#include <iostream.h>
#include <fstream.h>
#endif
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <tr1/unordered_map>
#include <limits.h>

#include "validator.h"
#include "alphabet.h"
#include "regexpr.h"
#include "tree.h"
#include "error.h"
#include "glushkov.h"
#include <xercesc/framework/XMLValidator.hpp>
#include <xercesc/validators/DTD/DTDValidator.hpp>
#include <xercesc/util/XMLString.hpp>
#include "regexpr.h"
#include "tree.h"
#include "str.h"

//XERCES_CPP_NAMESPACE_USE

struct hashXMLChArray : public std::unary_function<const XMLCh*,size_t>
{
   size_t operator()(const XMLCh* v) const 
   {
      return XMLString::hash(v,INT_MAX);
   };
};

struct equalXMLChArray : public std::binary_function<const XMLCh*,const XMLCh*,size_t>
{
   bool operator()(const XMLCh* v1,const XMLCh* v2) const
   {
      return XMLString::equals(v1,v2);
   };
};

template<class T> class hash_map : 
   public std::tr1::unordered_map<const XMLCh*,T,hashXMLChArray,equalXMLChArray>
{};

class StatValidator : public XMLValidator
{
   
public:
   Grammar* grammar;
   hash_map<int> symbol_id;
   hash_map<String> symbol;
   hash_map<Recognizer*> recognizer;
   hash_map<char*> def;
   hash_map<RegExpr*> regexpr;
   hash_map<Glushkov*> glushkov;
   hash_map<Tree*> tree;
   hash_map<Glushkov::Stats*> stats;

   String& getSymbol(const XMLCh* element_name);
   RegExpr::Node* translate(const ContentSpecNode* spec, Alphabet* alphabet);
   RegExpr* translateContentSpec(const ContentSpecNode* spec);
   Recognizer* translateMixedContentSpec(const ContentSpecNode* spec);
   Alphabet* gatherSymbols(const ContentSpecNode* spec, Alphabet* alphabet);

public:
   StatValidator(XMLErrorReporter* const errReporter = NULL);
   ~StatValidator();
   bool checkContent(XMLElementDecl*, QName**, XMLSize_t, XMLSize_t*);
   void faultInAttr(XMLAttr&, const XMLAttDef&) const;
   void preContentValidation(bool, bool);
   void postParseValidation();
   void reset();
   bool requiresNamespaces() const;
   void validateAttrValue(const XMLAttDef*, const XMLCh*, bool, const XMLElementDecl*);
   void validateElement(const XMLElementDecl*);
   Grammar* getGrammar() const;
   void setGrammar(Grammar*);
   bool handlesDTD() const;
   bool handlesSchema() const;
};




XERCES_CPP_NAMESPACE_USE

static char* CONTENT_TYPE[41] = {
   "Leaf"
   , "ZeroOrOne"
   , "ZeroOrMore"
   , "OneOrMore"
   , "Choice"
   , "Sequence"
   , "Any"
   , "Any_Other"
   , "Any_NS"
   , "All"
   , "Loop"
   , "(unk)"
   , "(unk)"
   , "(unk)"
   , "(unk)"
   , "(unk)"
   , "(unk)"
   , "(unk)"
   , "(unk)"
   , "(unk)"
   , "Any_NS_Choice"
   , "ModelGroupSequence"
   , "Any_Lax"
   , "Any_Other_Lax"
   , "Any_NS_Lax" 
   , "(unk)"
   , "(unk)"
   , "(unk)"
   , "(unk)"
   , "(unk)"
   , "(unk)"
   , "(unk)"
   , "(unk)"
   , "(unk)"
   , "(unk)"
   , "(unk)" 
   , "ModelGroupChoice"
   , "(unk)"
   , "Any_Skip"
   , "Any_Other_Skip"
   , "Any_NS_Skip"
};


StatValidator::StatValidator(XMLErrorReporter* const errReporter) :
   XMLValidator(errReporter)
{
   this->grammar = NULL;
}

StatValidator::~StatValidator()
{
}

bool StatValidator::checkContent(XMLElementDecl* decl, 
                             QName** children, 
                             XMLSize_t count, 
                             XMLSize_t* indexFailingChild)
{
   
   const XMLCh* element_name = decl->getFullName();
   if (recognizer[element_name] == NULL) {
      validateElement(decl);
   }

   return true;
}

void StatValidator::faultInAttr(XMLAttr& attr, const XMLAttDef& attr_def) const
{
}

void StatValidator::preContentValidation(bool, bool)
{
}

void StatValidator::postParseValidation()
{
}

void StatValidator::reset()
{
}

bool StatValidator::requiresNamespaces() const
{
   return false;
}

void StatValidator::validateAttrValue(const XMLAttDef*, const XMLCh*, bool, const XMLElementDecl*)
{
}

String& StatValidator::getSymbol(const XMLCh* element_name) 
{
   if (symbol[element_name].null()) {
      String s(XMLString::transcode(element_name));
      symbol[element_name] = s;
   }
      
   return symbol[element_name];
}

RegExpr::Node* StatValidator::translate(const ContentSpecNode* spec, Alphabet* alphabet)
{
   const XMLCh* element_name;

   switch (spec->getType()) {
   case ContentSpecNode::Leaf :
      element_name = spec->getElement()->getRawName();
      if (XMLString::stringLen(element_name) == 0) {
         // Corresponds to #PCDATA in a mixed content
         return new RegExpr::Node(RegExpr::EPSILON);
      } else {
         return new RegExpr::Node(RegExpr::LABEL,
                                  alphabet->find_id(getSymbol(element_name)));
      }
      break;
   case ContentSpecNode::Sequence :
      return new RegExpr::Node(RegExpr::CONCAT,
                               translate(spec->getFirst(),alphabet),
                               translate(spec->getSecond(),alphabet));
      break;
   case ContentSpecNode::Choice :
      return new RegExpr::Node(RegExpr::UNION,
                               translate(spec->getFirst(),alphabet),
                               translate(spec->getSecond(),alphabet));
      break;
   case ContentSpecNode::ZeroOrOne :
      return new RegExpr::Node(RegExpr::OPTION,
                               translate(spec->getFirst(),alphabet), NULL);
      break;
   case ContentSpecNode::OneOrMore :
      return new RegExpr::Node(RegExpr::PLUS,
                               translate(spec->getFirst(),alphabet),
                               NULL);
      break;
   case ContentSpecNode::ZeroOrMore :
      return new RegExpr::Node(RegExpr::STAR,
                               translate(spec->getFirst(),alphabet),
                               NULL);
      break;
   default:
      ERROR("Unhandled content specification type %s",CONTENT_TYPE[spec->getType()]);
      break;
   }
   return NULL;
}

Alphabet* StatValidator::gatherSymbols(const ContentSpecNode* spec, Alphabet* alphabet)
{
   const XMLCh* element_name;

   switch (spec->getType()) {
   case ContentSpecNode::Leaf :
      element_name = spec->getElement()->getRawName();
      if (XMLString::stringLen(element_name) != 0) {
         alphabet->find_id(getSymbol(element_name));
      }
      break;
   case ContentSpecNode::Choice :
      gatherSymbols(spec->getFirst(),alphabet);
      gatherSymbols(spec->getSecond(),alphabet);
      break;
   case ContentSpecNode::ZeroOrOne :
      gatherSymbols(spec->getFirst(),alphabet);
      break;
   case ContentSpecNode::OneOrMore :
      gatherSymbols(spec->getFirst(),alphabet);
      break;
   case ContentSpecNode::ZeroOrMore :
      gatherSymbols(spec->getFirst(),alphabet);
      break;
   case ContentSpecNode::Sequence :
   default:
      ERROR("Unhandled content specification type %s",CONTENT_TYPE[spec->getType()]);
      break;
   }

   return alphabet;
}

RegExpr* StatValidator::translateContentSpec(const ContentSpecNode* spec)
{
   Alphabet* alphabet = new Alphabet();
   return new RegExpr(alphabet,translate(spec,alphabet));
}

void StatValidator::validateElement(const XMLElementDecl* decl)
{
   const XMLCh* element_name = decl->getFullName();

   if (!stats[element_name]) {
      DTDElementDecl* d = (DTDElementDecl*) decl;

      //DEBUG("Parsing %s",XMLString::transcode(d->getFormattedContentModel()));
      
      switch (d->getModelType()) {
      case DTDElementDecl::Empty :
         recognizer[element_name] = Recognizer::EMPTY_WORD_ONLY;
         stats[element_name] = new Glushkov::Stats();
         stats[element_name]->empty = true;
         stats[element_name]->det = true;
         break;
      case DTDElementDecl::Any :
         recognizer[element_name] = Recognizer::UNIVERSAL;
         stats[element_name] = new Glushkov::Stats();
         stats[element_name]->any = true;
         stats[element_name]->det = true;
         break;
      case DTDElementDecl::Mixed_Simple :
      case DTDElementDecl::Children :
         regexpr[element_name] = translateContentSpec(d->getContentSpec());
         glushkov[element_name] = new Glushkov(regexpr[element_name]);
         stats[element_name] = glushkov[element_name]->gatherStats();
         stats[element_name]->mixed = d->getModelType() == DTDElementDecl::Mixed_Simple;
         break;
      case DTDElementDecl::ModelTypes_Count :
         ERROR("Unhandled DTD model type 'ModelTypes_Count'");
      }
      stats[element_name]->elem_name = XMLString::transcode(element_name);
      stats[element_name]->def = XMLString::transcode(d->getFormattedContentModel());
   }
}

Grammar* StatValidator::getGrammar() const
{
   return this->grammar;
}

void StatValidator::setGrammar(Grammar* grammar)
{
      this->grammar = grammar;
}

bool StatValidator::handlesDTD() const
{
   return true;
}

bool StatValidator::handlesSchema() const
{
   return false;
}


namespace {
XERCES_CPP_NAMESPACE_USE
// ---------------------------------------------------------------------------
//  This is a simple class that lets us do easy (though not terribly efficient)
//  trancoding of XMLCh data to local code page for display.
// ---------------------------------------------------------------------------
class StrX
{
public :
    // -----------------------------------------------------------------------
    //  Constructors and Destructor
    // -----------------------------------------------------------------------
    StrX(const XMLCh* const toTranscode)
    {
        // Call the private transcoding method
        fLocalForm = XMLString::transcode(toTranscode);
    }

    ~StrX()
    {
        XMLString::release(&fLocalForm);
    }

    // -----------------------------------------------------------------------
    //  Getter methods
    // -----------------------------------------------------------------------
    const char* localForm() const
    {
        return fLocalForm;
    }

   const char* str() const 
   {
      return fLocalForm;
   }

private :
    // -----------------------------------------------------------------------
    //  Private data members
    //
    //  fLocalForm
    //      This is the local code page form of the string.
    // -----------------------------------------------------------------------
    char*   fLocalForm;
};

inline XERCES_STD_QUALIFIER ostream& operator<<(XERCES_STD_QUALIFIER ostream& target, const StrX& toDump)
{
    target << toDump.localForm();
    return target;
}

class SAXCountHandlers : public HandlerBase
{
public:
   XMLSize_t       fAttrCount;
   XMLSize_t       fCharacterCount;
   XMLSize_t       fElementCount;
   XMLSize_t       fSpaceCount;
   bool            fSawErrors;
   hash_map<int>   count;
   char*           root_name;
public:
   // -----------------------------------------------------------------------
   //  Constructors and Destructor
   // -----------------------------------------------------------------------
   SAXCountHandlers() :

      fAttrCount(0)
      , fCharacterCount(0)
      , fElementCount(0)
      , fSpaceCount(0)
      , fSawErrors(false)
      , root_name(NULL)
   {
   }

   ~SAXCountHandlers()
   {
   }


   // -----------------------------------------------------------------------
   //  Getter methods
   // -----------------------------------------------------------------------
   XMLSize_t getElementCount() const
   {
      return fElementCount;
   }

   XMLSize_t getAttrCount() const
   {
      return fAttrCount;
   }

   XMLSize_t getCharacterCount() const
   {
      return fCharacterCount;
   }
   
   bool getSawErrors() const
   {
      return fSawErrors;
   }
   
   XMLSize_t getSpaceCount() const
   {
      return fSpaceCount;
   }

   // -----------------------------------------------------------------------
   //  Handlers for the SAX DocumentHandler interface
   // -----------------------------------------------------------------------
   void startElement(const XMLCh* const name, AttributeList& attributes)
   {
      if (root_name == NULL) {
         root_name = XMLString::transcode(name);
      }
      count[name]++;
      fElementCount++;
      fAttrCount += attributes.getLength();
   }

   void characters(const XMLCh* const chars, const XMLSize_t length)
   {
      fCharacterCount += length;
   }

   void ignorableWhitespace(const XMLCh* const chars, const XMLSize_t length)
   {
      fSpaceCount += length;
   }

   void resetDocument()
   {
      fAttrCount = 0;
      fCharacterCount = 0;
      fElementCount = 0;
      fSpaceCount = 0;
   }

    // -----------------------------------------------------------------------
    //  Handlers for the SAX ErrorHandler interface
    // -----------------------------------------------------------------------
	void warning(const SAXParseException& e)
   {
      XERCES_STD_QUALIFIER cerr 
         << "\nWarning at file " << StrX(e.getSystemId())
         << ", line " << e.getLineNumber()
         << ", char " << e.getColumnNumber()
         << "\n  Message: " << StrX(e.getMessage()) 
         << XERCES_STD_QUALIFIER endl;
   }

   void error(const SAXParseException& e)
   {
      fSawErrors = true;
      XERCES_STD_QUALIFIER cerr 
         << "\nError at file " << StrX(e.getSystemId())
         << ", line " << e.getLineNumber()
         << ", char " << e.getColumnNumber()
         << "\n  Message: " << StrX(e.getMessage()) 
         << XERCES_STD_QUALIFIER endl;
   }

   void fatalError(const SAXParseException& e)
   {
      fSawErrors = true;
      XERCES_STD_QUALIFIER cerr 
         << "\nFatal Error at file " << StrX(e.getSystemId())
         << ", line " << e.getLineNumber()
         << ", char " << e.getColumnNumber()
         << "\n  Message: " << StrX(e.getMessage()) 
         << XERCES_STD_QUALIFIER endl;
   }

   void resetErrors()
   {
      fSawErrors = false;
   }

};
}

void test();
// ---------------------------------------------------------------------------
//  Program entry point
// ---------------------------------------------------------------------------
int main(int argC, char* argV[])
{

   //Logger::getLogger()->level = Logger::INFO_LEVEL;
   //test();
   //exit(1);
   
   // Check command line and extract arguments.
   if (argC < 4) {
      XERCES_STD_QUALIFIER cout << "\nUsage:\n" 
                                << "   " << argV[0] << " <file_name> <dtd_id> <xml_id>"
                                << XERCES_STD_QUALIFIER endl;
      return 1;
   }
   
   const char*              xmlFile = argV[1];
   long                     dtd_id = atol(argV[2]);
   long                     xml_id = atol(argV[3]);
   SAXParser::ValSchemes    valScheme = SAXParser::Val_Auto;
   bool                     doNamespaces       = false;
   bool                     doSchema           = true;
   bool                     schemaFullChecking = false;
   bool                     errorOccurred = false;

   // Initialize the XML4C2 system
   try {
      XMLPlatformUtils::Initialize();
   } catch (const XMLException& toCatch) {
      XERCES_STD_QUALIFIER cerr 
         << "Error during initialization! Message:\n"
         << StrX(toCatch.getMessage()) 
         << XERCES_STD_QUALIFIER endl;
      return 1;
   }
   
   //
   //  Create a SAX parser object. Then, according to what we were told on
   //  the command line, set it to validate or not.
   //
   StatValidator* v = new StatValidator();
   SAXParser* parser;
   parser = new SAXParser(v);
   v->setErrorReporter(parser);
   parser->setValidationScheme(valScheme);
   parser->setDoNamespaces(doNamespaces);
   parser->setDoSchema(doSchema);
   parser->setHandleMultipleImports(true);
   parser->setValidationSchemaFullChecking(schemaFullChecking);
   
   SAXCountHandlers handler;
   parser->setDocumentHandler(&handler);
   parser->setErrorHandler(&handler);
   
   try {
      parser->parse(xmlFile);
   } catch (const OutOfMemoryException&) {
      XERCES_STD_QUALIFIER cerr << "OutOfMemoryException" << XERCES_STD_QUALIFIER endl;
      errorOccurred = true;
   } catch (const XMLException& e) {
      XERCES_STD_QUALIFIER cerr 
         << "\nError during parsing: '" << xmlFile << "'\n"
         << "Exception message is:  \n"
         << StrX(e.getMessage()) << "\n" << XERCES_STD_QUALIFIER endl;
      errorOccurred = true;
   } catch (...) {
      XERCES_STD_QUALIFIER cerr 
         << "\nUnexpected exception during parsing: '" << xmlFile << "'\n";
      errorOccurred = true;
   }


//    printf("DTD\n");
//    printf("states, init, edges, det, sigma, max_occur, avg_occur, "
//           "min_occur, mixed, nullable, empty, any, star, element_name, def\n");
   if (! handler.fSawErrors && ! errorOccurred) {
      hash_map<Glushkov::Stats*>::const_iterator i = v->stats.begin();
      while (i != v->stats.end()) {
         Glushkov::Stats* stats = i->second;
         printf("INSERT OR REPLACE INTO DTD VALUES "
                "(%li, \"%s\", \"%s\", %i, %i, %i, %i, %i, %i, %.2f,"
                " %i, %i, %i, %i, %i, %i, %i, 0);\n",
                dtd_id,
                stats->elem_name,
                stats->def,
                stats->state,
                stats->init,
                stats->edge,
                stats->det,
                stats->sigma,
                stats->max_occur,
                stats->avg_occur,
                stats->min_occur,
                stats->alternat,
                stats->mixed,
                stats->nullable,
                stats->empty,
                stats->any,
                stats->closure);
         i++;
      }
      printf("INSERT OR IGNORE INTO XML VALUES " 
             "(%li, %li, \"%s\", %li, %li, %li, 0);\n",
             xml_id,
             dtd_id,
             handler.root_name,
             handler.fElementCount,
             handler.fCharacterCount,
             handler.fAttrCount
             );
   } else {
      printf("INSERT OR IGNORE INTO DTD VALUES "
             "(%li, \"\", NULL, NULL, NULL, NULL,"
             " NULL, NULL, NULL, NULL, NULL,"
             " NULL, NULL, NULL, NULL, NULL, NULL, 1);\n",
             dtd_id);
      printf("INSERT OR IGNORE INTO XML VALUES " 
             "(%li, %li, NULL, NULL, NULL, NULL, 1);\n",
             xml_id,
             dtd_id);
   }
   //
   //  Delete the parser itself.  Must be done prior to calling Terminate, below.
   //
   delete parser;
   
   // And call the termination method
   XMLPlatformUtils::Terminate();
   
   return 0;
}


// int main(int argC, char* argV[])
// {

//    if (argC < 2) {
//       printf("Usage: \n"
//              "     stats <reg-expr>\n");
//    }
//    String re_def(argV[1],true);

//    RegExpr* e = RegExpr::parse_str(re_def);
//    Glushkov* g = new Glushkov(e);
//    Glushkov::Stats* stats = g->gatherStats();
//    printf("edges, states, det, sigma, max_occur, avg_occur, min_occur, mixed, nullable\n");
//    printf("%5i, %6i, %3i, %5i, %9i, %9.2f, %9i, %5i, %8i\n",
//           stats->edge,
//           stats->state,
//           stats->det,
//           stats->sigma,
//           stats->max_occur,
//           stats->avg_occur,
//           stats->min_occur,
//           stats->mixed,
//           stats->nullable);
// }
