//
// This file is part of LiRE, a free, experimental library for
// efficient testing of determinism and membership for regular
// expressions.
//
// LiRE is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, version 3 of the License, or any
// later version.
//
// LiRE is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LiRE. If not, see <http://www.gnu.org/licenses/>.
//
// Authors: Sławek Staworko <staworko@gmail.com>
//          Benoît Groz <benoit.groz@yahoo.fr>
//          Sebastian Maneth <sebastian.maneth@gmail.com>
// Date: 20 June 2011.
//
#ifndef LiRE_STR_H_
#define LiRE_STR_H_

#include "vector.h"

/**
 * Abstract string representation. 
 */
struct String
{

   static String Null;
   static String getCanonicalString(long int i);

   char* str;
   int*  ref_count;
   int   size;

   String();
   String(long int ptr);
   String(char* str, bool copy = false);
   String(const char* str);
   String(const String& s);
   ~String();

   void purge();

   void init();
   void init(const char* str);
   void init(char* str, bool copy = false);
   void init(const String& s);

   void reinit();
   void reinit(const char* str);
   void reinit(char* str, bool copy = false);
   void reinit(const String& s);

   bool null() const;
   int  len() const;

   char& operator[](int i);
   String& operator=(const String& s);
   String& operator+=(const String& s);
   char* print() const;
};

String operator+(const String& s1, const String s2);
bool operator==(const String& s1, const String& s2);


#endif // LiRE_STR_H_
