#include "str.h"
#include "alphabet.h"
#include "regexpr.h"
#include "glushkov.h"
#include "tree.h"

#include <stdio.h>
#include <time.h>


void print_vector(Vector<int>& v);
template<class T, void(*print_T)(T&) > 
void _print_vector(Vector<T>& v)
{
   if (v.null()) {
      printf("NULL");
   } else {
      printf("[");
      for (int i = 0; i < v.size; i++) {
         print_T(v[i]);
         if (i < v.size - 1) {
            printf(", ");
         }
      }
      printf("]");
   }
}
void print_NodePtr(Node*& p)
{
   if (p == NULL) {
      printf("(null)");
   } else {
      printf("%i",p->id);
   }
}

void print_vector(Vector<Node*>& v)
{
   _print_vector<Node*,print_NodePtr>(v);
}

void print_bool(bool& b);
void print_int(int& b);
void print_vector(Vector<bool>& v);

void print_vector(Vector<int>& v);
void print_vector(Vector<bool>& v);
void print_vector(Vector<Vector<bool> >& v);
void print_vector(Vector<Vector<int> >& v)
{
   _print_vector<Vector<int>,_print_vector<int,print_int> >(v);
}


#define IS_BRANCH_TOP(n) (n->parent == NULL or n->parent->left != n)
#define IS_SEGMENT_TOP(n) ((n->is_root_last()) or IS_BRANCH_TOP(n))
#define IS_SEGMENT_BOTTOM(n) (n->is_epsilon() or n->is_pos() or n->left->is_root_last())

namespace {
void assign_n_ref(Tree* t, 
                  Node* n, 
                  Node* n_ref,
                  Node* y)
{ 
   if (n == NULL) {
      return;
   }
   
   if (n->is_pos()) {
      if (n->is_root_first()) {
         n_ref = n->parent->left;
      }
      ASSERT(t->n_ref[n->pos_id] == n_ref, " at position %i being %i",
             n->pos_id,
             n_ref->id);
      return;
   } else {
      if (n->is_root_first() and not n->nullable) {
         n_ref = n->parent->left;
      }
      if (n->is_root_first() and n->nullable) {
         n_ref = y;
      }
      if (n->is_root_last() or 
          n->is_root_first() or 
          (IS_BRANCH_TOP(n) and n->nullable) or 
          (n->parent->is_union() and n->parent->right == n)) {
         y = n;
      }
   }

   assign_n_ref(t,n->left,n_ref,y);
   assign_n_ref(t,n->right,n_ref,y);
}

void assign_n_ref_root(Tree* t)
{
   if (t->root->right->is_pos()) {
      ASSERT(t->n_ref[t->root->right->pos_id] == t->root);
   }

   assign_n_ref(t,t->root->right->left,
                t->root,t->root->right);
   assign_n_ref(t,t->root->right->right,
                t->root,t->root->right);
}


void assign_next_top(Tree* t, Node* n, Node* y, Node* y_cand, bool in_seg_y_cand)
{
   if (n == NULL) {
      return;
   }

   if (n->is_epsilon()) {
      ASSERT(t->next_top[n->id] == NULL);
      return;
   }

   if (n->is_pos() || IS_SEGMENT_TOP(n)) {
      if (n->is_root_first() and not n->nullable) {
         ASSERT(t->next_top[n->id] == n->parent->left);
      } else {
         ASSERT(t->next_top[n->id] == y,"For node %i with y being %i instead of %i",
                n->id,(y)?(y->id):(-1),(t->next_top[n->id])?(t->next_top[n->id]->id):(-1));
      }
      if (n->is_pos()) {
         return;
      }
   } else {
      ASSERT(t->next_top[n->id] == NULL);
   }

   if (n->is_root_first() or n->is_root_last()) {
      y = n;
      y_cand = NULL;
   } 

   if (y_cand != NULL and not n->nullable and n->is_concat() and in_seg_y_cand) {
      y = y_cand;
   }

   if (IS_BRANCH_TOP(n) and (n->nullable or n->parent->is_union())) {
      y_cand = n;
      if (not n->nullable and n->is_concat()) {
         y = y_cand;
      }
   }
   
   assign_next_top(t,n->left,y,y_cand,not IS_SEGMENT_TOP(n->left));
   assign_next_top(t,n->right,y,y_cand,false);
}


void assign_next_top_root(Tree* t)
{
   ASSERT(t->next_top[t->root->id] == NULL);
   
   Node* n = t->root->right;

   if (n->is_epsilon()) {
      ASSERT(t->next_top[n->id] == NULL);
      return;
   }

   if (n->is_pos() or IS_SEGMENT_TOP(n)) {
      ASSERT(t->next_top[n->id] == t->root);
   }

   if (n->is_pos()) {
      return;
   }

   Node* y = n;
   Node* y_cand = NULL;

   if (IS_BRANCH_TOP(n) and n->nullable) {
      y_cand = n;
   }

   assign_next_top(t,n->left,y,y_cand,not IS_SEGMENT_TOP(n->left));
   assign_next_top(t,n->right,y,y_cand,false);
}

}

int main()
{
   unsigned seed = (unsigned) time(NULL);
   seed = 1322350161;
   srandom(seed);
   printf("Seed is %u\n",seed);
//    RegExpr* e = RegExpr::parse_str("(0|(1,(2,(3)*)))");

//    for (int j = 0; j < 10000; j++) {
//    do { 
//       int size = random_int(100) + 1;
//       e = RegExpr::random_re(size, 2*size);
//       printf("x");
//    } while (!(new Tree(e))->is_deterministic());
//    printf(".");
//    fflush(stdout);

//    PlusFreeRecognizer* r = new PlusFreeRecognizer(e);
//    e->print();
//    Vector<bool> nullable(r->t->size);
//    for (int i = 0; i < r->t->size; i++) {
//       nullable[i] = r->t->get_node(i)->nullable;
//    }
//    printf("RootFirst: ");
//    print_vector(r->t->root_first);
//    printf("\n");
//    printf("RootLast: ");
//    print_vector(r->t->root_last);
//    printf("\n");
//    printf("Nref: ");
//    print_vector(r->t->n_ref);
//    printf("\n");
//    printf("Nullable: ");
//    print_vector(nullable);
//    printf("\n");
//    printf("NextTop: ");
//    print_vector(r->t->next_top);
//    printf("\n");

//    assign_n_ref_root(r->t);
//    assign_next_top_root(r->t);

//    }


//    Vector<bool> nullable(r->t->size);
//    for (int i = 0; i < r->t->size; i++) {
//       nullable[i] = r->t->get_node(i)->nullable;
//    }
//    printf("RootFirst: ");
//    print_vector(r->t->root_first);
//    printf("\n");
//    printf("RootLast: ");
//    print_vector(r->t->root_last);
//    printf("\n");
//    printf("Nref: ");
//    print_vector(r->t->n_ref);
//    printf("\n");
//    printf("Nullable: ");
//    print_vector(nullable);
//    printf("\n");
//    unsigned seed = (unsigned) time(NULL);
//    //seed = 1322332372;
//    srandom(seed);
//    printf("Random seed is %u\n",seed);
//    RegExpr* e = RegExpr::parse_str("a?,(EMPTY|b*)");
//    e->print();
//    PlusFreeRecognizer* r = new PlusFreeRecognizer(e);

//    Vector<int> s(4);
//    s[0] = 1;
//    s[1] = 1;
//    s[2] = 1;
//    s[3] = 1;
//    printf("Input word: ");
//    print_vector(s);
//    printf("\n");
//    printf("RootFirst: ");
//    print_vector(r->t->root_first);
//    printf("\n");
//    printf("RootLast: ");
//    print_vector(r->t->root_last);
//    printf("\n");
//    printf("Nref: ");
//    print_vector(r->t->n_ref);
//    printf("\n");

//    ASSERT(r->match(s));

   for (int i = 0 ; i < 10000; i++) {
      int size = random_int(50) + 1;
      RegExpr* e = RegExpr::random_re(size, 2.5*size);
      //e->print();
      Tree* t = new Tree(e);
      if (t->is_deterministic()) {
         DetGlushkovRecognizer* r = new DetGlushkovRecognizer(e);
         Recognizer* g = new GlushkovRecognizer(e);
         printf("o");
         fflush(stdout);
         for (int j = 0 ; j < 1000; j ++) {
            Vector<int> s(random_int(100) + 1);
            e->random_matching_word(s);
            //print_vector(s);
            bool res = r->match(s);
            //printf("\nMatch: %i\n",res);
            if (!res) {
               printf("\nNo Match for: ");
               e->print();
               print_vector(s);
               printf("Init: ") ;
               print_vector(r->init_state);
               printf("\nDelta: ");
               print_vector(r->delta_table);
               printf("\nFinal: ");
               print_vector(r->final);
               printf("\n");
               exit(0);
            }
            for (int k = 0; k < s.size; k++) {
               s[k] = random_int(e->alphabet->size);
            }
            
            if (r->match(s) != g->match(s)) {
               printf("\nWrong Match for:");
               e->print();
               print_vector(s);
               printf("Glushkov gives %i\n",g->match(s));
               printf("Plus-free gives %i\n",r->match(s));
               exit(0);
            }

            //printf(".");
         }
         //delete r;
      } else {
         printf("x");
         fflush(stdout);
      }
      //delete t;
      //delete e;
   }
}
