//
// This file is part of LiRE, a free, experimental library for
// efficient testing of determinism and membership for regular
// expressions.
//
// LiRE is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, version 3 of the License, or any
// later version.
//
// LiRE is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LiRE. If not, see <http://www.gnu.org/licenses/>.
//
// Authors: Sławek Staworko <staworko@gmail.com>
//          Benoît Groz <benoit.groz@yahoo.fr>
//          Sebastian Maneth <sebastian.maneth@gmail.com>
// Date: 20 June 2011.
//
#ifndef LiRE_USKEL_H_
#define LiRE_USKEL_H_

#include "vector.h"
#include "lca.h"

struct Node;
typedef Vector<Node*> NVector;

struct USkel
{
   struct Node
   {
      int id;
      int depth;
      
      int in_id;
      int out_id;
      
      ::Node* node;
      
      Node* parent;
      Node* left;   // first child.
      Node* right;  // next sibling.
      
      Node();
      
      ~Node();
      
      void print();
      void print(int ind);
      void _print(int ind);
   };
   
   Node*         root;
   Vector<Node*> us_node;
   Vector<Node*> in_node;
   Node*         in_null;
   int size;
   LCA<USkel,Node>* lca;

   USkel(NVector& node, Vector<int>& node_id);
   ~USkel();

   void build_uskel(NVector& node, Vector<int>& node_id);
   Node* add_new_node(Node* anchor, ::Node* n, int node_id, int& id);
   Node* roll_back(Node* anchor, int& id);
   Node* get_node(int id);

   Node* get_lca(Node* x, Node* y);
   //Vector<Node*>* getRootPath(Node* x);
   int getHighest(int id_1, int id_2);

   void print();
};

#endif // LiRE_USKEL_H_
