//
// This file is part of LiRE, a free, experimental library for
// efficient testing of determinism and membership for regular
// expressions.
//
// LiRE is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, version 3 of the License, or any
// later version.
//
// LiRE is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LiRE. If not, see <http://www.gnu.org/licenses/>.
//
// Authors: Sławek Staworko <staworko@gmail.com>
//          Benoît Groz <benoit.groz@yahoo.fr>
//          Sebastian Maneth <sebastian.maneth@gmail.com>
// Date: 20 June 2011.
//

#include <string.h>
#include <stdio.h>
#include <math.h>

#include "utils.h"


// For reference and testing of the optimized version only .
char log64_ref(uint64_t x)
{
   ASSERT(x > 0);

   register int r = 0;

   while (x >>= 1) {
      r++;
   }

   return r;
}

// For reference and testing of the optimized version only .
uint64_t exp64_ref(int i)
{
   uint64_t res = 1;
   for (int j = 0; j < i; j++) {
      res *= 2;
   }

   return res;
}
// Returns an integer whose binary representation has the bits 0
// through i-1 set to 1, and the remaining bits set to 0.
extern uint64_t bit_mask_ref(int length)
{
   ASSERT(0 <= length && length <= 64);

   uint64_t res = 0;
   for (int j = 0; j < length; j++) {
      res <<= 1;
      res |= 1;
   }

   return res;
}

extern uint64_t sqrt_ref(uint64_t i)
{
   return ceil(sqrt((float) i));
}

// Returns a random integer from '0' to 'bound-1'.
unsigned random_int(unsigned bound)
{
   ASSERT(bound > 0,"Non-positive bound given %i",bound);

   return random() % bound;
}

// Returns a random long integer from '0' to 'bound-1'. 
unsigned long random_long(unsigned long bound)
{
   ASSERT(bound > 0,"Non-positive bound given %lu",bound);

   unsigned long chunk1 = random();
   unsigned long chunk2 = random();

   unsigned long rand_long = (chunk1 << 32) | chunk2;

   return rand_long % bound;
}

// Returns a random Boolean. 
bool random_bool()
{
   return random() & 0x01;
}

char* canonical_label(long id)
{
   static char* _buf = new char[41];

   sprintf(_buf,"a%li",id);

   return _buf;
}

size_t str_len(const char* str)
{
   const char* tmp = str;

   while (*tmp != '\0') {
      tmp++;
   }
   
   return static_cast<size_t>(tmp - str);
}

const char* str_dup(const char* str)
{
   char* res = new char[str_len(str)+1];
   char* tmp = res;

   while (*str != '\0') {
      *tmp++ = *str++;
   }

   return res;
}

void indent(int ind)
{
   for (int i = 0; i < ind; i++) {
      printf(" "); 
   }
}

void latex_indent(int ind)
{
   for (int i = 0; i < ind; i++) {
      printf("\\>"); 
   }
}

