//
// This file is part of LiRE, a free, experimental library for
// efficient testing of determinism and membership for regular
// expressions.
//
// LiRE is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, version 3 of the License, or any
// later version.
//
// LiRE is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LiRE. If not, see <http://www.gnu.org/licenses/>.
//
// Authors: Sławek Staworko <staworko@gmail.com>
//          Benoît Groz <benoit.groz@yahoo.fr>
//          Sebastian Maneth <sebastian.maneth@gmail.com>
// Date: 20 June 2011.
//
#ifndef LiRE_VALIDATOR_H_
#define LiRE_VALIDATOR_H_

#include <xercesc/framework/XMLValidator.hpp>
#include <xercesc/validators/DTD/DTDValidator.hpp>
#include <xercesc/util/XMLString.hpp>
#include "regexpr.h"
#include "tree.h"
#include "str.h"
#include "timer.h"
#include "vector.h"

XERCES_CPP_NAMESPACE_USE

class Validator : public XMLValidator
{
   struct hashXMLChArray : public std::unary_function<const XMLCh*,size_t>
   {
      size_t operator()(const XMLCh* v) const 
      {
         //Timer::Stop();
         return XMLString::hash(v,INT_MAX);
         //Timer::Start();
      };
   };
   
   struct equalXMLChArray : public std::binary_function<const XMLCh*,const XMLCh*,size_t>
   {
      bool operator()(const XMLCh* v1,const XMLCh* v2) const
      {
         //Timer::Stop();
         return XMLString::equals(v1,v2);
         //Timer::Start();
      };
   };
   
   template<class T> class hash_map : 
      public std::tr1::unordered_map<const XMLCh*,T,hashXMLChArray,equalXMLChArray>
   {};
   
private:
   Grammar* grammar;
   RecognizerFactory* f;
   hash_map<int> symbol_id;
   hash_map<String> symbol;
   hash_map<Recognizer*> recognizer;

   String& getSymbol(const XMLCh* element_name);
   RegExpr::Node* translate(const ContentSpecNode* spec, Alphabet* alphabet);
   RegExpr* translateContentSpec(const ContentSpecNode* spec);
   Recognizer* translateMixedContentSpec(const ContentSpecNode* spec);
   void gatherAndTranslateSymbols(const ContentSpecNode* spec, List<String>& alphabet);
   void gatherAndTranslateSymbols(const ContentSpecNode* spec, Vector<String>& alphabet);

public:
   Validator(RecognizerFactory* f, XMLErrorReporter* const errReporter = NULL);
   ~Validator();
   bool checkContent(XMLElementDecl*, QName**, XMLSize_t, XMLSize_t*);
   void faultInAttr(XMLAttr&, const XMLAttDef&) const;
   void preContentValidation(bool, bool);
   void postParseValidation();
   void reset();
   bool requiresNamespaces() const;
   void validateAttrValue(const XMLAttDef*, const XMLCh*, bool, const XMLElementDecl*);
   void validateElement(const XMLElementDecl*);
   Grammar* getGrammar() const;
   void setGrammar(Grammar*);
   bool handlesDTD() const;
   bool handlesSchema() const;
};



#endif // LiRE_VALIDATOR_H_
