#include "gtest/gtest.h"
#include "vector.h"
#include "unit.h"

TEST(CollectionsT,Vector) {
   Vector<int> v(4);

   v.fill(0);
   EXPECT_EQ(0,v[0]);
   EXPECT_EQ(0,v[1]);
   EXPECT_EQ(0,v[2]);
   EXPECT_EQ(0,v[3]);
   EXPECT_EQ(4,v.size);

   v[0] = 1;
   v[1] = 2;
   v[2] = 3;
   v[3] = 4;

   EXPECT_ASSERTION_FAILURE(v[-1]);
   EXPECT_EQ(1,v[0]);
   EXPECT_EQ(2,v[1]);
   EXPECT_EQ(3,v[2]);
   EXPECT_EQ(4,v[3]);
   EXPECT_ASSERTION_FAILURE(v[4]);
   EXPECT_EQ(4,v.size);
   EXPECT_FALSE(v.empty());
   EXPECT_FALSE(v.null());

   Vector<int> u(v,1,2);

   EXPECT_ASSERTION_FAILURE(u[-1]);
   EXPECT_EQ(2,u[0]);
   EXPECT_EQ(3,u[1]);
   EXPECT_ASSERTION_FAILURE(u[2]);
   EXPECT_EQ(2,u.size);
   EXPECT_FALSE(u.empty());
   EXPECT_FALSE(u.null());

   u.reinit();   
   EXPECT_EQ(-1,u.size);
   EXPECT_TRUE(u.null());
   EXPECT_TRUE(u.empty());
   EXPECT_ASSERTION_FAILURE(u[0]);

   v.purge();   
   EXPECT_EQ(-1,v.size);
   EXPECT_TRUE(v.null());
   EXPECT_TRUE(v.empty());
   EXPECT_ASSERTION_FAILURE(v[0]);

   v.reinit(0);
   EXPECT_EQ(0,v.size);
   EXPECT_FALSE(v.null());
   EXPECT_TRUE(v.empty());
   EXPECT_ASSERTION_FAILURE(v[0]);
}

TEST(CollectionsT,List) {
   List<int> l;
   
   EXPECT_EQ(0,l.size);
   EXPECT_TRUE(l.empty());
   EXPECT_EQ(0,l.add(1));
   EXPECT_EQ(1,l.size);
   EXPECT_FALSE(l.empty());
   EXPECT_EQ(1,l.add(3));
   EXPECT_EQ(2,l.size);
   EXPECT_FALSE(l.empty());
   EXPECT_EQ(2,l.add(5));
   EXPECT_EQ(3,l.size);
   EXPECT_FALSE(l.empty());
   EXPECT_EQ(3,l.add(7));
   EXPECT_EQ(4,l.size);
   EXPECT_FALSE(l.empty());
   EXPECT_EQ(4,l.add(9));
   EXPECT_EQ(5,l.size);
   EXPECT_FALSE(l.empty());

   Vector<int> v(l);

   EXPECT_EQ(5,v.size);
   EXPECT_EQ(1,v[0]);
   EXPECT_EQ(3,v[1]);
   EXPECT_EQ(5,v[2]);
   EXPECT_EQ(7,v[3]);
   EXPECT_EQ(9,v[4]);

   Vector<int> v1;

   l.pack_into(v1);
   EXPECT_EQ(5,v1.size);
   EXPECT_EQ(1,v1[0]);
   EXPECT_EQ(3,v1[1]);
   EXPECT_EQ(5,v1[2]);
   EXPECT_EQ(7,v1[3]);
   EXPECT_EQ(9,v1[4]);
}

TEST(CollectionsT,Queue) {
   Queue<int> q;

   EXPECT_TRUE(q.empty());
   EXPECT_EQ(0,q.size);
   EXPECT_EQ(0,q.add(1));
   EXPECT_EQ(1,q.first());
   EXPECT_EQ(1,q.size);
   EXPECT_EQ(1,q.add(2));
   EXPECT_EQ(1,q.first());
   EXPECT_EQ(2,q.size);
   EXPECT_EQ(2,q.add(3));
   EXPECT_EQ(1,q.first());
   EXPECT_EQ(3,q.size);
   EXPECT_EQ(3,q.add(4));
   EXPECT_EQ(1,q.first());
   EXPECT_EQ(4,q.size);
   EXPECT_FALSE(q.empty());
   
   EXPECT_EQ(1,q.remove());
   EXPECT_EQ(2,q.first());
   EXPECT_EQ(3,q.size);
   EXPECT_EQ(2,q.remove());
   EXPECT_EQ(3,q.first());
   EXPECT_EQ(2,q.size);
   EXPECT_FALSE(q.empty());

   EXPECT_EQ(2,q.add(0));
   EXPECT_EQ(3,q.first());   
   EXPECT_EQ(3,q.size);
   EXPECT_FALSE(q.empty());

   EXPECT_EQ(3,q.remove());
   EXPECT_EQ(4,q.first());   
   EXPECT_EQ(2,q.size);
   EXPECT_EQ(4,q.remove());
   EXPECT_EQ(0,q.first());   
   EXPECT_EQ(1,q.size);
   EXPECT_EQ(0,q.remove());
   EXPECT_EQ(0,q.size);
   EXPECT_TRUE(q.empty());
   EXPECT_ASSERTION_FAILURE(q.first());

   EXPECT_TRUE(q.empty());
   EXPECT_EQ(0,q.size);
   EXPECT_EQ(0,q.add(1));
   EXPECT_EQ(1,q.size);
   EXPECT_EQ(1,q.add(2));
   EXPECT_EQ(2,q.size);
   EXPECT_EQ(2,q.add(3));
   EXPECT_EQ(3,q.size);
   EXPECT_EQ(3,q.add(4));
   EXPECT_EQ(4,q.size);
   EXPECT_FALSE(q.empty());
   
   EXPECT_EQ(1,q.remove());
   EXPECT_EQ(3,q.size);
   EXPECT_EQ(2,q.remove());
   EXPECT_EQ(2,q.size);
   EXPECT_FALSE(q.empty());

   EXPECT_EQ(2,q.add(0));
   EXPECT_EQ(3,q.size);
   EXPECT_FALSE(q.empty());

   EXPECT_EQ(3,q.remove());
   EXPECT_EQ(2,q.size);
   EXPECT_EQ(4,q.remove());
   EXPECT_EQ(1,q.size);
   EXPECT_EQ(0,q.remove());
   EXPECT_EQ(0,q.size);
   EXPECT_TRUE(q.empty());
}

TEST(CollectionsT,Stack) {
   Stack<int> s;

   EXPECT_TRUE(s.empty());
   EXPECT_ASSERTION_FAILURE(s.pop());
   EXPECT_ASSERTION_FAILURE(s.peek());

   s.push(1);
   s.push(2);
   s.push(3);

   EXPECT_EQ(3,s.size);

   EXPECT_FALSE(s.empty());
   EXPECT_EQ(3,s.peek());
   EXPECT_EQ(3,s.size);
   EXPECT_EQ(3,s.pop());
   EXPECT_EQ(2,s.size);
   EXPECT_EQ(2,s.peek());
   EXPECT_EQ(2,s.size);
   EXPECT_EQ(2,s.pop());
   EXPECT_EQ(1,s.size);
   EXPECT_EQ(1,s.peek());
   EXPECT_EQ(1,s.size);
   EXPECT_EQ(1,s.pop());
   EXPECT_EQ(0,s.size);
   EXPECT_TRUE(s.empty());
}

TEST(CollectionsT,Array) {

}

TEST(CollectionsT,HArray) {

}

