#include "lca.h"
#include "gtest/gtest.h"
#include "utils.h"
#include "logger.h"

namespace {
struct Tree
{
   struct Node
   {
      int id;
      int depth;
      Node* parent;
      Node* left;
      Node* right;

      Node(Node* left, Node* right)
      {
         this->id = -1;
         this->depth = -1;
         this->parent = NULL;
         this->left = left;
         this->right = right;
      };

      ~Node()
      {
         if (this->left != NULL) {
            delete this->left;
         }
         if (this->right != NULL) {
            delete this->right;
         }
      };
      
      int size()
      {
         int left_size = (this->left != NULL) ? 
            (this->left->size()) : (0);
         int right_size = (this->right != NULL) ? 
            (this->right->size()) : (0);

         return 1 + left_size + right_size;
      };
   };

   Node* root;
   Vector<Node*> node;
   int size;

   Tree(Node* root)
   {
      this->root = root;
      this->size = root->size();
      node.reinit(this->size);
      int id = 0;
      traverse(root,NULL,id,0);
      
   };

   ~Tree()
   {
      delete this->root;
   };

   void traverse(Node* n, Node* parent, int& id, int depth)
   {
      if (n == NULL) {
         return;
      }

      n->id = id;
      n->depth = depth;
      n->parent = parent;
      this->node[id] = n;
      id++;

      traverse(n->left,n,id,depth+1);
      traverse(n->right,n,id,depth+1);
   };

   Node* get_node(int id)
   {
      ASSERT(0 <= id && id < this->size);

      return node[id];
   };

   void getRootPath(Node* n, Vector<Node*>& path)
   {
      path.reinit(n->depth+1);
      do {
         path[n->depth] = n;
         n = n->parent;
      } while (n != NULL);
   };

   Node* get_lca(Node* x, Node * y)
   {
      Vector<Node*> path_x;
      Vector<Node*> path_y;

      getRootPath(x,path_x);
      getRootPath(y,path_y);

      int min_depth = min(x->depth,y->depth);

      int i = 0;
      while (i+1 <= min_depth && path_x[i+1] == path_y[i+1]) {
         i++;
      }

      return path_x[i];
   };

   // the (choice) ration between unary and binary nodes.
   static Node* _random_tree(int size)
   {
      ASSERT(size != 0);
      if (size == 1) {
         return new Node(NULL,NULL);
      } else if (size == 2) {
         return new Node(new Node(NULL,NULL),NULL);
      } else if (random_int(100) <= 12) {
         return new Node(_random_tree(size-1),NULL);
      } else {
         int split = 1 + random_int(size-2);
         return new Node(_random_tree(split),
                         _random_tree(size-1-split));
      }
   }
   static Tree* random_tree(int size)
   {
      ASSERT(size > 0);
      return new Tree(_random_tree(size));
   }
};

typedef Tree::Node Node;
}; // namespace

TEST(LowestCommonAncestorT,MockTest) {
   Node* root = new Node(new Node(new Node(NULL,NULL),NULL),
                         new Node(new Node(NULL,NULL),
                                  new Node(NULL,NULL)));

   Tree* t = new Tree(root);

   Node* n0 = t->get_node(0);

   Node* n1 = t->get_node(1);
   Node* n2 = t->get_node(2);
   Node* n3 = t->get_node(3);
   Node* n4 = t->get_node(4);
   Node* n5 = t->get_node(5);

   EXPECT_EQ(0,t->get_lca(n0,n0)->id);
   EXPECT_EQ(0,t->get_lca(n0,n1)->id);
   EXPECT_EQ(0,t->get_lca(n0,n2)->id);
   EXPECT_EQ(0,t->get_lca(n0,n3)->id);
   EXPECT_EQ(0,t->get_lca(n0,n4)->id);
   EXPECT_EQ(0,t->get_lca(n0,n5)->id);
   
   EXPECT_EQ(0,t->get_lca(n1,n0)->id);
   EXPECT_EQ(1,t->get_lca(n1,n1)->id);
   EXPECT_EQ(1,t->get_lca(n1,n2)->id);
   EXPECT_EQ(0,t->get_lca(n1,n3)->id);
   EXPECT_EQ(0,t->get_lca(n1,n4)->id);
   EXPECT_EQ(0,t->get_lca(n1,n5)->id);
   
   EXPECT_EQ(0,t->get_lca(n2,n0)->id);
   EXPECT_EQ(1,t->get_lca(n2,n1)->id);
   EXPECT_EQ(2,t->get_lca(n2,n2)->id);
   EXPECT_EQ(0,t->get_lca(n2,n3)->id);
   EXPECT_EQ(0,t->get_lca(n2,n4)->id);
   EXPECT_EQ(0,t->get_lca(n2,n5)->id);
   
   EXPECT_EQ(0,t->get_lca(n3,n0)->id);
   EXPECT_EQ(0,t->get_lca(n3,n1)->id);
   EXPECT_EQ(0,t->get_lca(n3,n2)->id);
   EXPECT_EQ(3,t->get_lca(n3,n3)->id);
   EXPECT_EQ(3,t->get_lca(n3,n4)->id);
   EXPECT_EQ(3,t->get_lca(n3,n5)->id);
   
   EXPECT_EQ(0,t->get_lca(n4,n0)->id);
   EXPECT_EQ(0,t->get_lca(n4,n1)->id);
   EXPECT_EQ(0,t->get_lca(n4,n2)->id);
   EXPECT_EQ(3,t->get_lca(n4,n3)->id);
   EXPECT_EQ(4,t->get_lca(n4,n4)->id);
   EXPECT_EQ(3,t->get_lca(n4,n5)->id);
   
   EXPECT_EQ(0,t->get_lca(n5,n0)->id);
   EXPECT_EQ(0,t->get_lca(n5,n1)->id);
   EXPECT_EQ(0,t->get_lca(n5,n2)->id);
   EXPECT_EQ(3,t->get_lca(n5,n3)->id);
   EXPECT_EQ(3,t->get_lca(n5,n4)->id);
   EXPECT_EQ(5,t->get_lca(n5,n5)->id);

   delete t;
}

TEST(LowestCommonAncestorT,FixedInstance) {

   Node* root = new Node(new Node(new Node(NULL,NULL),NULL),
                         new Node(new Node(NULL,NULL),
                                  new Node(NULL,NULL)));

   Tree* t = new Tree(root);
   LCA<Tree,Node>* lca = new LCA<Tree,Node>(t);

   Node* n0 = t->get_node(0);
   Node* n1 = t->get_node(1);
   Node* n2 = t->get_node(2);
   Node* n3 = t->get_node(3);
   Node* n4 = t->get_node(4);
   Node* n5 = t->get_node(5);

   EXPECT_EQ(0,lca->get_lca(n0,n0)->id);
   EXPECT_EQ(0,lca->get_lca(n0,n1)->id);
   EXPECT_EQ(0,lca->get_lca(n0,n2)->id);
   EXPECT_EQ(0,lca->get_lca(n0,n3)->id);
   EXPECT_EQ(0,lca->get_lca(n0,n4)->id);
   EXPECT_EQ(0,lca->get_lca(n0,n5)->id);
   
   EXPECT_EQ(0,lca->get_lca(n1,n0)->id);
   EXPECT_EQ(1,lca->get_lca(n1,n1)->id);
   EXPECT_EQ(1,lca->get_lca(n1,n2)->id);
   EXPECT_EQ(0,lca->get_lca(n1,n3)->id);
   EXPECT_EQ(0,lca->get_lca(n1,n4)->id);
   EXPECT_EQ(0,lca->get_lca(n1,n5)->id);
   
   EXPECT_EQ(0,lca->get_lca(n2,n0)->id);
   EXPECT_EQ(1,lca->get_lca(n2,n1)->id);
   EXPECT_EQ(2,lca->get_lca(n2,n2)->id);
   EXPECT_EQ(0,lca->get_lca(n2,n3)->id);
   EXPECT_EQ(0,lca->get_lca(n2,n4)->id);
   EXPECT_EQ(0,lca->get_lca(n2,n5)->id);
   
   EXPECT_EQ(0,lca->get_lca(n3,n0)->id);
   EXPECT_EQ(0,lca->get_lca(n3,n1)->id);
   EXPECT_EQ(0,lca->get_lca(n3,n2)->id);
   EXPECT_EQ(3,lca->get_lca(n3,n3)->id);
   EXPECT_EQ(3,lca->get_lca(n3,n4)->id);
   EXPECT_EQ(3,lca->get_lca(n3,n5)->id);
   
   EXPECT_EQ(0,lca->get_lca(n4,n0)->id);
   EXPECT_EQ(0,lca->get_lca(n4,n1)->id);
   EXPECT_EQ(0,lca->get_lca(n4,n2)->id);
   EXPECT_EQ(3,lca->get_lca(n4,n3)->id);
   EXPECT_EQ(4,lca->get_lca(n4,n4)->id);
   EXPECT_EQ(3,lca->get_lca(n4,n5)->id);

   EXPECT_EQ(0,lca->get_lca(n5,n0)->id);
   EXPECT_EQ(0,lca->get_lca(n5,n1)->id);
   EXPECT_EQ(0,lca->get_lca(n5,n2)->id);
   EXPECT_EQ(3,lca->get_lca(n5,n3)->id);
   EXPECT_EQ(3,lca->get_lca(n5,n4)->id);
   EXPECT_EQ(5,lca->get_lca(n5,n5)->id);

   delete t;
   delete lca;
}


TEST(LowestCommonAncestorT,RandomInstance) {
   int size = 100 + random_int(1000);
   Tree* t = Tree::random_tree(size);
   LCA<Tree,Node>* lca = new LCA<Tree,Node>(t);
   
   for (int i = 0; i < 10000; i++) {
      Node* n1 = t->get_node(random_int(size));
      Node* n2 = t->get_node(random_int(size));

      EXPECT_EQ(t->get_lca(n1,n2),lca->get_lca(n1,n2));
   }
}
