#include "vector.h"
#include "utils.h"
#include "glushkov.h"
#include "tree.h"
#include "gtest/gtest.h"

void print_int(int& i);

void print_vector(Vector<int>& v);



struct suite {
   char* re;
   char* pos[255];
   char* neg[255];
} A[] = 
   {
      {"a",
       {"a", "a", NULL},
       {"aa",NULL}},
      {"a|b",
       {"a", "b", NULL},
       {"ab", NULL}},
      {"EMPTY",
       {"", NULL},
       {"a", "b", NULL}},
      {"a|EMPTY",
       {"a", "", NULL},
       {"aa", NULL}},
      {"(a,(EMPTY|b),c)*",
       {"","a b c", "a b c a b c", "a c a b c a c", NULL},
       {"a", "a b", "a b b c", "a c b", NULL}},
      {NULL,
       {NULL},
       {NULL}}
   };


TEST(MatchingT,FixedRegExpr) {
   for(int i = 0; A[i].re != NULL; i++) {
      String re(A[i].re,true);

      RegExpr* e = RegExpr::parse_str(re);
      Glushkov* g = new Glushkov(e);
      //Tree* t = new Tree(e);
      
      for (int j = 0; A[i].pos[j] != NULL; j++) {
         Vector<int> symbol_id;

         String pos(A[i].pos[j],true);
         e->alphabet->scan_str(pos,symbol_id);

//          if (!g->match(symbol_id)) {
//             DEBUG("RE: %s, pos: %s", A[i].re, A[i].pos[j]);
//             g->print();
//             printf("Symbol seq: ");
//             print_vector(symbol_id);
//             printf("\n");
//          }

         EXPECT_TRUE(g->match(symbol_id));
//          if (t->is_deterministic()) {
//             EXPECT_TRUE(t->match(symbol_id));
//          }
      }

      for (int j = 0; A[i].neg[j] != NULL; j++) {
         Vector<int> symbol_id;

         String neg(A[i].neg[j],true);
         e->alphabet->scan_str(neg,symbol_id);

//          if (g->match(symbol_id)) {
//             DEBUG("RE: %s, neg: %s", A[i].re, A[i].neg[j]);
//             g->print();
//             printf("Symbol seq: ");
//             print_vector(symbol_id);
//             printf("\n");
//          }
         EXPECT_FALSE(g->match(symbol_id));
//          if (t->is_deterministic()) {
//             EXPECT_FALSE(t->match(symbol_id));
//          }
      }

      delete e;
      delete g;
      //delete t;
   }
}

TEST(MatchingT,RandomRegExpr) {
   for (int i = 0; i < 10; i++) {
      RegExpr* e = RegExpr::random_re(120,750);

      Tree* t = new Tree(e);

      if (t->is_deterministic()) {
         //printf("*DET");
         DetTreeRecognizer* r = new DetTreeRecognizer(e);
         int word_size = random_int(10000);
         Vector<int> token_id(word_size);

         e->random_matching_word(token_id);

         EXPECT_TRUE(r->match(token_id));
         delete r;
      } else {
         //printf("*NON");
      }
      delete e;
      delete t;
   }
}

