#include <string.h>

#include "regexpr.h"
#include "utils.h"
#include "gtest/gtest.h"

namespace {
void serialize(RegExpr::Node* n, Vector<String>& a, List<String>& out)
{
   switch (n->type) {
   case RegExpr::EPSILON:
      out.add("EMPTY");
      break;
   case RegExpr::LABEL:
      out.add(a[n->label_id]);
      break;
   case RegExpr::STAR:
      out.add("(");
      serialize(n->left,a,out);
      out.add(")*");
      break;
   case RegExpr::CONCAT:
      out.add("(");
      serialize(n->left,a,out);
      out.add(",");
      serialize(n->right,a,out);
      out.add(")");
      break;
   case RegExpr::UNION:
      out.add("(");
      serialize(n->left,a,out);
      out.add("|");
      serialize(n->right,a,out);
      out.add(")");
      break;
   case RegExpr::OPTION:
      out.add("(");
      serialize(n->left,a,out);
      out.add(")?");
      break;
   default:
      ERROR("Unhandled ReGExpr::NType");
   }
}

String to_str(RegExpr* e)
{
   Vector<String> a;
   Queue<String> out;

   e->alphabet->pack_into(a);
   serialize(e->root,a,out);

   String res;

   while (! out.empty()) {
      res += out.remove();
   }
   
   return res;
}

bool equals(RegExpr::Node* n, RegExpr::Node* m, 
            Vector<String>& a, Vector<String>& b)
{
   if (n->type != m->type) {
      return false;
   } else if (n->type == RegExpr::LABEL) {
      return a[n->label_id] == b[m->label_id];
   } else if (n->left != NULL && m->left == NULL) {
      return false;
   } else if (n->left == NULL && m->left != NULL) {
      return false;
   } else if (n->left != NULL && !equals(n->left,m->left,a,b)) {
      return false;
   } else if (n->right != NULL && m->right == NULL) {
      return false;
   } else if (n->right == NULL && m->right != NULL) {
      return false;
   } else if (n->right != NULL && !equals(n->right,m->right,a,b)) {
      return false;
   } else {
      return true;
   }
}

bool equals(RegExpr* e1, RegExpr* e2)
{
   Vector<String> a, b;

   e1->alphabet->pack_into(a);
   e2->alphabet->pack_into(b);

   return equals(e1->root,e2->root,a,b);
}

}; // namespace

TEST(ParsingT,Random) {
   for (int i = 0; i < 100; i++) {
      int size = 1 + random_int(1000);
      int label_count = 1 + random_int(100);
      RegExpr* e = RegExpr::random_re(size,label_count);
      
      String str = to_str(e);
      
      RegExpr* eb = RegExpr::parse_str(str);
      
      EXPECT_TRUE(equals(e,eb));

      delete e;
      delete eb;
   }
}

