#include "rmq.h"
#include "gtest/gtest.h"


using testing::Types;

// Implementation of RMQ interface
typedef Types<RMQ,NaiveRMQ,ST_RMQ> RMQImplementations;
typedef Types<RMQ,NaiveRMQ,ST_RMQ,PM_RMQ> PMRMQImplementations;


template <class RMQImpl>
class PMRMQStaticTest : public testing::Test {
protected:
   Vector<int> a;
   RMQImpl* rmq;
   PMRMQStaticTest() : a(10) 
   {
      a[0] = 2; a[1] = 3; a[2] = 1; a[3] = 4; a[4] = 1;
      a[5] = 5; a[6] = 0; a[7] = 1; a[8] = 0; a[9] = 5;
      rmq = new RMQImpl(a);
   }

   virtual ~PMRMQStaticTest() 
   { 
      delete rmq; 
   }
};

template <class RMQImpl>
class RMQBisymTest : public testing::Test {
protected:
   Vector<int> a;
   RMQ* rmq_ref;
   RMQImpl* rmq;

   RMQBisymTest() 
   {
      int length = random_int(1000)+100;
      a.reinit(length);

      for (int i = 0; i < length; i++) {
         a[i] = random_int(20000);
      }

      rmq_ref = new RMQ(a);
      rmq = new RMQImpl(a);
   }

   virtual ~RMQBisymTest() 
   { 
      delete rmq_ref;
      delete rmq; 
   }
};

template <class RMQImpl>
class PMRMQBisymTest : public testing::Test {
protected:
   Vector<int> a;
   RMQ* rmq_ref;
   RMQImpl* rmq;

   PMRMQBisymTest() 
   {
      int length = random_int(1000)+100;
      a.reinit(length);
      a[0] = random_int(20000);

      for (int i = 1; i < length; i++) {
         a[i] = a[i-1] + (random_bool()?(+1):(-1));
      }

      rmq_ref = new RMQ(a);
      rmq = new RMQImpl(a);
   }

   virtual ~PMRMQBisymTest() 
   { 
      delete rmq_ref;
      delete rmq; 
   }
};

TYPED_TEST_CASE(PMRMQStaticTest,PMRMQImplementations);
TYPED_TEST_CASE(RMQBisymTest,RMQImplementations);
TYPED_TEST_CASE(PMRMQBisymTest,PMRMQImplementations);

TYPED_TEST(PMRMQStaticTest,FixedInstance) {
   EXPECT_EQ(0,this->rmq->query(0,0));
   EXPECT_EQ(0,this->rmq->query(0,1));
   EXPECT_EQ(2,this->rmq->query(0,2));
   EXPECT_EQ(2,this->rmq->query(2,3));
   EXPECT_EQ(6,this->rmq->query(3,6));
   EXPECT_EQ(this->a[2],this->a[this->rmq->query(2,5)]);
   EXPECT_EQ(6,this->rmq->query(4,7));
   EXPECT_EQ(4,this->rmq->query(3,5));
   EXPECT_EQ(this->a[6],this->a[this->rmq->query(5,9)]);
   EXPECT_EQ(this->a[6],this->a[this->rmq->query(1,9)]);
}

TYPED_TEST(RMQBisymTest,RandomQueries) {
   for (int iter = 0; iter < 2000; iter++) {
      int i = random_int(this->a.size);
      int j = random_int(this->a.size-i)+i;
      EXPECT_EQ(this->a[this->rmq_ref->query(i,j)],
                this->a[this->rmq->query(i,j)]);
   }
}

TYPED_TEST(PMRMQBisymTest,RandomQueries) {
   for (int iter = 0; iter < 2000; iter++) {
      int i = random_int(this->a.size);
      int j = random_int(this->a.size-i)+i;
      EXPECT_EQ(this->a[this->rmq_ref->query(i,j)],
                this->a[this->rmq->query(i,j)]);
   }
}

