#include "utils.h"
#include "gtest/gtest.h"

TEST(UtilsT,RandomInt) {
   for (int i = 1; i < 20; i+=5) {
      int j = random_int(i);
      EXPECT_LE(0,j);
      EXPECT_LT(j,i);
   }
}

TEST(UtilsT,RandomLong) {
   for (int i = 1; i < 200; i+=50) {
      int j = random_long(i);
      EXPECT_LE(0,j);
      EXPECT_LT(j,i);
   }
}

TEST(UtilsT,MinMax) {
   long x = random_int(100);
   long y = random_int(100);
   
   long min_val = min(x,y);
   long max_val = max(x,y);

   EXPECT_LE(min_val,max_val);
   EXPECT_LE(min_val,x);
   EXPECT_LE(min_val,y);
   EXPECT_GE(max_val,x);
   EXPECT_GE(max_val,y);
   EXPECT_TRUE(x == max_val || y == max_val);
   EXPECT_TRUE(x == min_val || y == min_val);
}

TEST(UtilsT,AbsValue) {
   EXPECT_EQ(0,abs(0));
   EXPECT_EQ(1,abs(1));
   EXPECT_EQ(1,abs(-1));
}

TEST(UtilsT,Log) {
   EXPECT_EQ(0,log8(1));
   EXPECT_EQ(1,log8(2));
   EXPECT_EQ(1,log8(3));
   EXPECT_EQ(2,log8(4));
   EXPECT_EQ(2,log8(5));

   for(int bit = 0; bit < 8; bit++) {
      uint64_t val = 1 << bit;
      val += random_int(val);
      EXPECT_EQ(bit,log8(val));
   }

   for(int bit = 0; bit < 16; bit++) {
      uint64_t val = 1 << bit;
      val += random_int(val);
      EXPECT_EQ(bit,log16(val));
   }

   for(int bit = 0; bit < 32; bit++) {
      uint64_t val = 1 << bit;
      val += random_int(val);
      EXPECT_EQ(bit,log32(val));
   }

   for(int bit = 0; bit < 64; bit++) {
      uint64_t val = ((uint64_t)1) << bit;
      val += random_long(val);
      EXPECT_EQ(bit,log64(val)) << " for val = " << val;
   }

   for (int i = 0; i < 20; i++) {
      uint64_t v = random_long(UINT64_MAX-1)+1;
      EXPECT_EQ(log64(v),log64_ref(v));
   }
   
}


TEST(UtilsT,Exp) {
   for (int i= 0; i < 31; i++) {
      EXPECT_EQ(((uint64_t)exp(i)),exp64_ref(i));
   }
}

TEST(UtilsT,Swap) {
   int a = 1;
   int b = 3;
   swap(a,b);

   EXPECT_EQ(1,b);
   EXPECT_EQ(3,a);
}

// int main(int argc, char **argv) {
//   ::testing::InitGoogleTest(&argc, argv);
//   return RUN_ALL_TESTS();
// }
