#include "utils.h"
#include "gtest/gtest.h"
#include "unit.h"
#include "veb.h"

using testing::Types;

typedef Types<OrdMap<int>,VEB<int> > OrdMapImplementations;

template <class OrdMapImpl>
class OrdMapStaticTest : public testing::Test {
protected:
   OrdMapImpl* s;
   OrdMapStaticTest() 
   {
      s = new OrdMapImpl(50);
   }
   virtual ~OrdMapStaticTest() 
   { 
      delete s; 
   }
};

template <class OrdMapImpl>
class OrdMapRandomTest : public testing::Test {
protected:
   int domain;
   OrdMap<int>* s_ref;
   OrdMapImpl* s;
   OrdMapRandomTest() 
   {
      domain = 100+random_int(900);
      s = new OrdMapImpl(domain);
      s_ref = new OrdMap<int>(domain);
   }
   virtual ~OrdMapRandomTest() 
   { 
      delete s;
      delete s_ref;
   }
};

TYPED_TEST_CASE(OrdMapStaticTest,OrdMapImplementations);
TYPED_TEST_CASE(OrdMapRandomTest,OrdMapImplementations);

TYPED_TEST(OrdMapStaticTest,FixedInstance) {
   EXPECT_EQ(-1,this->s->find_pred(49));
   EXPECT_EQ(-1,this->s->find_succ(0));

   this->s->insert(10,1);
   this->s->insert(20,2);
   this->s->insert(30,1);

   EXPECT_EQ(3,this->s->size);

   EXPECT_EQ(10,this->s->find_succ(5));
   EXPECT_EQ(20,this->s->find_succ(15));
   EXPECT_EQ(30,this->s->find_succ(25));
   EXPECT_EQ(-1,this->s->find_succ(35));

   EXPECT_EQ(-1,this->s->find_pred(5));
   EXPECT_EQ(10,this->s->find_pred(15));
   EXPECT_EQ(20,this->s->find_pred(25));
   EXPECT_EQ(30,this->s->find_pred(35));

   EXPECT_EQ(1,this->s->get(10));
   EXPECT_EQ(2,this->s->get(20));
   EXPECT_EQ(1,this->s->get(30));

   EXPECT_EQ(int(),this->s->get(5));
   EXPECT_EQ(int(),this->s->get(15));
   EXPECT_EQ(int(),this->s->get(25));
   EXPECT_EQ(int(),this->s->get(35));
   EXPECT_EQ(int(),this->s->get(45));
}

TYPED_TEST(OrdMapRandomTest,RandomInstance) {

   for (int i = 0; i < 10; i++) {
      int e = random_int(this->domain);
      if (this->s_ref->contains(e)) {
         EXPECT_TRUE(this->s->contains(e));
      } else {
         int n = random_int(10000);
         this->s->insert(e,n);
         this->s_ref->insert(e,n);
      }
   }
   

   for (int i = -1; i <= this->domain; i++) {
      EXPECT_EQ(this->s_ref->get(i),this->s->get(i));
      EXPECT_EQ(this->s_ref->find_succ(i),this->s->find_succ(i));
      EXPECT_EQ(this->s_ref->find_pred(i),this->s->find_pred(i));
   }
}

